#
# spec file for package containerd
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


#Compat macro for new _fillupdir macro introduced in Nov 2017
%if ! %{defined _fillupdir}
  %define _fillupdir /var/adm/fillup-templates
%endif

# Handle _multibuild magic.
%define flavour %{nil}

# We split the Name: into "realname" and "name_suffix".
%define realname containerd
%if "%flavour" == ""
%define name_suffix %{nil}
%else
%define name_suffix -%{flavour}
%endif

# MANUAL: Update the git_version.
%define git_version 894b81a4b802e4eb2a91d1ce216b8817763c29fb
%define git_short   894b81a4b802

Name:           %{realname}%{name_suffix}
Version:        1.2.6
Release:        lp151.2.6.1
Summary:        Standalone OCI Container Daemon
License:        Apache-2.0
Group:          System/Management
Url:            https://containerd.tools
Source:         %{realname}-%{version}_%{git_short}.tar.xz
Source1:        %{realname}-rpmlintrc
# OPENSUSE-FIX-UPSTREAM: Backport of https://github.com/containerd/containerd/pull/2764.
Patch1:         0001-makefile-remove-emoji.patch
BuildRequires:  fdupes
BuildRequires:  glibc-devel-static
BuildRequires:  go-go-md2man
BuildRequires:  libbtrfs-devel >= 3.8
BuildRequires:  libseccomp-devel >= 2.2
BuildRequires:  pkg-config
BuildRequires:  golang(API) >= 1.11
# We provide a git revision so that Docker can require it properly.
Provides:       %{name}-git = %{git_version}
# Currently runc is the only supported runtime for containerd. We pin the same
# flavour as us, to avoid mixing (the version pinning is done by docker.spec).
Requires:       docker-runc%{name_suffix}
Requires(post): %fillup_prereq
ExcludeArch:    s390
# KUBIC-SPECIFIC: This was required when upgrading from the original kubic
#                 packaging, when everything was renamed to -kubic. It also is
#                 used to ensure that nothing complains too much when using
#                 -kubic packages. Hopfully it can be removed one day.
%if "%flavour" == "kubic"
# Obsolete older package without -kubic suffix: v2 -> v3
Obsoletes:      %{realname} = 0.2.5+gitr569_2a5e70c
Obsoletes:      %{realname}_2a5e70c
# Conflict with non-kubic package, and provide equivalent
Conflicts:      %{realname} > 0.2.5+gitr569_2a5e70c
Provides:       %{realname} = %{version}
# Disable leap based builds for kubic flavor (bsc#1121412)
%if 0%{?suse_version} == 1500 && 0%{?is_opensuse}
ExclusiveArch:  do_not_build
%endif
%endif

%description
Containerd is a daemon with an API and a command line client, to manage
containers on one machine. It uses runC to run containers according to the OCI
specification. Containerd has advanced features such as seccomp and user
namespace support as well as checkpoint and restore for cloning and live
migration of containers.

%package ctr
Summary:        Client for %{name}
Group:          System/Management
Requires:       %{name} = %{version}
# KUBIC-SPECIFIC: This was required when upgrading from the original kubic
#                 packaging, when everything was renamed to -kubic. It also is
#                 used to ensure that nothing complains too much when using
#                 -kubic packages. Hopfully it can be removed one day.
%if "%flavour" == "kubic"
# Obsolete older package without -kubic suffix: v2 -> v3
Obsoletes:      %{realname}-ctr = 0.2.5+gitr569_2a5e70c
# Conflict with non-kubic package, and provide equivalent
Conflicts:      %{realname}-ctr > 0.2.5+gitr569_2a5e70c
Provides:       %{realname}-ctr = %{version}
%endif

%description ctr
Standalone client for containerd, which allows management of containerd containers
separately from Docker.

%prep
%setup -q -n %{realname}-%{version}_%{git_short}
%patch1 -p1

%build
# Do not use symlinks. If you want to run the unit tests for this package at
# some point during the build and you need to directly use go list directly it
# will get confused by symlinks.
export GOPATH=$HOME/go
export PROJECT=$HOME/go/src/github.com/containerd/containerd
mkdir -p $PROJECT
rm -rf $PROJECT/*
cp -ar * $PROJECT

BUILDTAGS="apparmor selinux seccomp"
make -C $PROJECT \
	BUILDTAGS="$BUILDTAGS" \
	VERSION="v%{version}" \
	REVISION="%{git_version}"
make man

cp $PROJECT/bin/ctr ctr-%{version}
cp $PROJECT/bin/containerd containerd-%{version}
cp $PROJECT/bin/containerd-shim containerd-shim-%{version}

%check
# We used to run 'go test' here, however we found that this actually didn't
# catch any issues that were caught by smoke testing, and %check would
# continually cause package builds to fail due to flaky tests. If you ever need
# to know how the testing was done, you can always look in the package history.
# boo#1095817

%install
# Install binaries.
install -D -m755 containerd-%{version} %{buildroot}/%{_sbindir}/%{realname}
install -D -m755 containerd-shim-%{version} %{buildroot}/%{_sbindir}/%{realname}-shim
install -D -m755 ctr-%{version} %{buildroot}/%{_sbindir}/%{realname}-ctr

# Install docker-* symlinks to said binaries, since in order to use the
# upstream setup, Docker needs to spawn containerd and needs to have the
# binaries have specific names.
ln -s %{_sbindir}/%{realname} %{buildroot}/%{_sbindir}/docker-%{realname}
ln -s %{_sbindir}/%{realname}-shim %{buildroot}/%{_sbindir}/docker-%{realname}-shim

# Set up dummy configuration.
install -d -m755 %{buildroot}/%{_sysconfdir}/%{realname}
echo "# See containerd-config.toml(5) for documentation." >%{buildroot}/%{_sysconfdir}/%{realname}/config.toml

# Man pages.
for file in man/*
do
	section="${file##*.}"
	install -D -m644 "$file" "%{buildroot}/%{_mandir}/man$section/$(basename "$file")"
done
ln -s ctr.1 %{buildroot}/%{_mandir}/man1/%{realname}-ctr.1

%fdupes %{buildroot}

%files
%defattr(-,root,root)
%doc README.md
%license LICENSE
%dir %{_sysconfdir}/%{realname}
%config %{_sysconfdir}/%{realname}/config.toml
%{_sbindir}/%{realname}
%{_sbindir}/docker-%{realname}
%{_sbindir}/%{realname}-shim
%{_sbindir}/docker-%{realname}-shim
%{_mandir}/man*/%{realname}*
%exclude %{_mandir}/man1/*ctr.1*

%files ctr
%{_sbindir}/%{realname}-ctr
%{_mandir}/man1/*ctr.1*

%changelog
* Fri Jun 28 2019 Aleksa Sarai <asarai@suse.com>
- Update to containerd v1.2.6, which is required for Docker v18.09.7-ce.
  bsc#1139649
- Remove containerd-test (it's not useful for actual testing).
* Fri May  3 2019 Aleksa Sarai <asarai@suse.com>
- Update to containerd v1.2.5, which is required for v18.09.5-ce.
  bsc#1128376 boo#1134068
  https://github.com/containerd/containerd/releases/tag/v1.2.5
* Thu Mar 21 2019 Sascha Grunert <sgrunert@suse.com>
- Update containerd to v1.2.4
  * cri: Set /etc/hostname
  * cri: Fix env performance issue
  * runc updated to 6635b4f0c6af3810594d2770f662f34ddc15b40d to solve
    bsc#1121967 CVE-2019-5736
  * cri updated to da0c016c830b2ea97fd1d737c49a568a816bf964
  * Windows: NewDirectIOFromFIFOSet
  * Changelogs from previous versions also included in this update:
    https://github.com/containerd/containerd/releases/tag/v1.2.3
* Tue Feb  5 2019 Aleksa Sarai <asarai@suse.com>
- Update to containerd v1.2.2, which is required for Docker v18.09.1-ce.
  bsc#1124308
  * Fix rare deadlock on FIFO creation with timeout
  * Fix a bug that a container can't be stopped or inspected when its
    corresponding image is deleted
  * Fix a bug that the cri plugin handles containerd events outside of
    k8s.io namespace
  more changes at:
  https://github.com/containerd/containerd/releases/tag/v1.2.2
  Changelogs from previous versions also included in this update:
    https://github.com/containerd/containerd/releases/tag/v1.2.1
    https://github.com/containerd/containerd/releases/tag/v1.2.0
    https://github.com/containerd/containerd/releases/tag/v1.1.4
    https://github.com/containerd/containerd/releases/tag/v1.1.3
- Remove required_dockerrunc commit pinning, as it just lead to issues.
- Remove upstreamed patches.
  - 0001-docs-man-rename-config.toml-5-to-be-more-descriptive.patch
* Fri Jan 11 2019 Sascha Grunert <sgrunert@suse.com>
- Disable leap based builds for kubic flavor. bsc#1121412
* Thu Dec 20 2018 clee@suse.com
- Update go requirements to >= go1.10 to fix
  * bsc#1118897 CVE-2018-16873
    go#29230 cmd/go: remote command execution during "go get -u"
  * bsc#1118898 CVE-2018-16874
    go#29231 cmd/go: directory traversal in "go get" via curly braces in import paths
  * bsc#1118899 CVE-2018-16875
    go#29233 crypto/x509: CPU denial of service
* Mon Nov  5 2018 Aleksa Sarai <asarai@suse.com>
- Add backport of https://github.com/containerd/containerd/pull/2764, which is
  required for us to build containerd on i586 SLE-12 (where /bin/sh doesn't
  like emoji in shell scripts). bsc#1102522 bsc#1113313
  + 0001-makefile-remove-emoji.patch
* Wed Aug 22 2018 asarai@suse.com
- Upgrade to containerd v1.1.2, which is required for Docker v18.06.1-ce.
  bsc#1102522
* Thu Aug 16 2018 asarai@suse.com
- Merge -kubic packages back into the main Virtualization:containers packages.
  This is done using _multibuild to add a "kubic" flavour, which is then used
  to conditionally compile patches and other kubic-specific features.
  bsc#1105000
* Wed Aug  1 2018 asarai@suse.com
- Enable seccomp support on SLE12, since libseccomp is now a new enough vintage
  to work with Docker and containerd. fate#325877
* Wed Jul 25 2018 asarai@suse.com
- Update to containerd v1.1.1, which is the required version for the Docker
  v18.06.0-ce upgrade. bsc#1102522
- Add backport of https://github.com/containerd/containerd/pull/2534 to make
  the man page no longer pollute the global namespace.
  + 0001-docs-man-rename-config.toml-5-to-be-more-descriptive.patch
- Remove the following patch since it has already been merged upstream.
  - bsc1065109-0001-makefile-add-support-for-build_flags.patch
- Remove systemd-related files and add docker-containerd-* symlinks; this
  aligns with the upstream defaults where dockerd will execute
  docker-containerd. Version upgrades of docker are expected to work more
  smoothly as much of the upgrade logic is implemented in dockerd.
- Add containerd-rpmlintrc (or containerd-kubic-rpmlintrc) to deal with
  /usr/src/containerd/* rpmlint errors (which don't affect normal users of this
  package).
* Wed Jun 13 2018 dcassany@suse.com
- Make use of %%license macro
* Tue Jun  5 2018 asarai@suse.com
- Remove 'go test' from %%check section, as it has only ever caused us problems
  and hasn't (as far as I remember) ever caught a release-blocking issue. Smoke
  testing has been far more useful. boo#1095817
* Wed May 16 2018 jmassaguerpla@suse.com
- Review obsoletes tag to fix bsc#1080978
* Thu Apr 12 2018 fcastelli@suse.com
- Put containerd under the podruntime slice. This the recommended
  deployment to allow fine resource control on Kubernetes.
  bsc#1086185
* Mon Feb 12 2018 rbrown@suse.com
- Add ${version} to equivalent non-kubic package provides
* Thu Feb  8 2018 rbrown@suse.com
- Add Provides for equivalent non-kubic packages
* Thu Feb  1 2018 ro@suse.de
- do not build on s390, only on s390x (no go on s390)
* Wed Dec 27 2017 dimstar@opensuse.org
- Fix build with RPM 4.14: exclude is not meant for files to NOT be
  packaged, but should only be used if the files are to be excluded
  from a glob when they end up in a different package. Rather
  remove the unwanted files in the install section.
* Mon Dec 11 2017 asarai@suse.com
- Update to containerd@06b9cb35161009dcb7123345749fef02f7cea8e0, which is
  requried by Docker 17.09.1_ce.
* Thu Nov 23 2017 rbrown@suse.com
- Replace references to /var/adm/fillup-templates with new
  %%_fillupdir macro (boo#1069468)
* Thu Oct 26 2017 asarai@suse.com
- Set --start-timeout=2m by default to match upstream. bsc#1064926
* Thu Oct 26 2017 asarai@suse.com
- Use the upstream makefile so that Docker can get the commit ID in `docker
  info`. This also will avoid possible future warnings being spit out like
  bsc#1065109 and boo#1053532.
- Backport https://github.com/containerd/containerd/pull/1686, which is
  required for the above fix. bsc#1065109 boo#1053532
  + bsc1065109-0001-makefile-add-support-for-build_flags.patch
* Wed Oct 18 2017 asarai@suse.com
- Update to containerd@3addd840653146c90a254301d6c3a663c7fd6429, which is
  required by Docker 17.07.0_ce (this commit is effectively v0.2.9 with a few
  bugfixes missing).
* Thu Aug  3 2017 asarai@suse.com
- Use -buildmode=pie for tests and binary build. bsc#1048046 bsc#1051429
* Thu Jul  6 2017 thipp@suse.de
- change dependency to docker-runc
* Thu May  4 2017 jmassaguerpla@suse.com
- fix golang requirement to 1.7 for the subpackages
* Tue May  2 2017 jmassaguerpla@suse.com
- fix golang requirement to 1.7
* Fri Apr 28 2017 jengelh@inai.de
- Replace %%__-type macro indirections
* Thu Apr 13 2017 jmassaguerpla@suse.com
- update containerd to the commit version needed for
  docker-v17.04.0-ce (bsc#1034053)
  fix bsc#1032769: containerd spurious messages filling journal
* Wed Apr 12 2017 jmassaguerpla@suse.com
- make sure this package is being built with go 1.7
* Wed Apr 12 2017 jmassaguerpla@suse.com
- remove the go_arches macro because we are using go1.7 which
  is available in all archs
* Wed Apr 12 2017 asarai@suse.com
- Set TasksMax=infinity to make sure runC doesn't start failing randomly.
* Fri Feb 24 2017 jmassaguerpla@suse.com
- update to docker 1.13.0 requirement
* Sat Jan 14 2017 asarai@suse.com
- Update docker to the version used in Docker 1.12.6. This is necessary to fix
  CVE-2016-9962 (bsc#1012568).
* Mon Dec 19 2016 jmassaguerpla@suse.com
- update containerd to the version used in docker 1.12.5 (bsc#1016307).
  This fixes bsc#1015661
* Thu Nov 24 2016 jmassaguerpla@suse.com
- fix runc version
  fix bsc#1009961
* Thu Nov 24 2016 jmassaguerpla@suse.com
- fix version so that it contains a sequence number and zypper does
  not think is a downgrade
* Tue Oct 25 2016 jmassaguerpla@suse.com
- fix bsc#1006368: docker/containerd is broken when installed by
  SuSE Studio in an appliance: We were missing the
    Requires(post): %%fillup_prereq
* Tue Oct 25 2016 jmassaguerpla@suse.com
- update runc requirement to 02f8fa7863dd3f82909a73e2061897828460d52f
  (see RUNC_COMMIT in Dockerfile)
* Thu Oct 13 2016 jmassaguerpla@suse.com
- update to commit 0366d7e which is the one required for docker-1.12.2
  (bsc#1004490)
* Mon Sep 19 2016 jmassaguerpla@suse.com
- fix go_arches definition: use global instead of define, otherwise
  it fails to build
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Remove GOPATH at the end of the GOPATH assignment
  cause GOPATH is empty and if we do that, we get the path ""
  appended, which causes gcc6-go to complain
* Wed Aug 24 2016 jmassaguerpla@suse.com
- add go_arches in project configuration: this way, we can use the
  same spec file but decide in the project configuration if to
  use gc-go or gcc-go for some archs.
* Mon Aug 22 2016 jmassaguerpla@suse.com
- update to v2.3.0 (bsc#995058)
- Remove patches which were already merged upstream:
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
* Thu Aug 18 2016 jmassaguerpla@suse.com
- use gcc6-go instead of gcc5-go (bsc#988408)
- build ppc64le with gc-go because this version builds with gc-go 1.6
* Thu Aug 18 2016 cbrauner@suse.com
- bump git commit id to the one required by docker v1.12.0
- run test during build
- only run tests on architectures that provide the go list and got test tools
- add aarch64 to go arches
* Thu Aug 18 2016 cbrauner@suse.de
- Add containerd-test package which contains the source code and the test. This
  package will be used to run the integration tests.
- Simplify package build and check sections: Instead of symlinking we default to
  cp -avr. go list gets confused by symlinks hence, we need to copy the source
  code anyway if we want to run unit tests during package build at some point.
* Tue Aug 16 2016 asarai@suse.com
  * Explicitly state the version dependency for runC, to avoid potential
  issues with incompatible component versions. These must be updated
  * each time we do a release*. Unfortunately we cannot create a hard
  dependency because that would conflict with Docker, and was a mistake
  on upstream's part. bsc#993847
* Wed Jul 20 2016 asarai@suse.com
  * Set --runtime option specifically to runC. bsc#978260
* Fri Jun 17 2016 asarai@suse.de
  * Update to containerd v0.2.2. (bsc#989566 FATE#320763)
  * Includes updates to the out-of-tree patches.
* Mon May  2 2016 asarai@suse.de
  * Remove MountFlags=slave from containerd.service. This causes many issues with
  interactions with Docker.
* Fri Apr 29 2016 asarai@suse.de
  * Added /usr/sbin/rccontainerd symlink as per suse-missing-rclink.
  * Updated socket activation patches to use the same patchset that was merged
  upstream (https://github.com/docker/containerd/pull/178):
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
  * Removed aarch64 that was patched upstream:
  - fix-aarch64-epoll.patch
  * Update containerd to 0.2.1. Upstream changelog:
  * Fixes for cgroup memory updates and process labeling.
  * Truncate the event log on disk and in memory so that it does not
    grow forever.  This is mainly used for higher levels to receive past
    events if they miss any.
* Tue Apr 19 2016 asarai@suse.de
  * Use the gc compiler for aarch64 builds.
  * Add a patch to fix the new aarch64 build support, which has not yet been
  merged upstream (https://github.com/docker/containerd/pull/195):
  + fix-aarch64-epoll.patch
  * Rebase the socket activation patchset which has yet to be merged
  (https://github.com/docker/containerd/pull/178):
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
  * Update to containerd 0.2.0. Changelog:
  + Add Limit to PidsStats
  + Add timeout flag for container start times.
  + Add timeout option for GRPC connection.
  + Add no_pivot_root support.
  + Add runtimeArgs to pass to shim
  * Move epoll syscall to a separate package so we can build on aarch64.
  * Fix ctr termios restoration isssues.
  * Several bug fixes.
  - Remove dependencies on larger packages.
* Thu Mar 31 2016 asarai@suse.de
  * Use socket activation with the containerd-daemon. This requires a
  not-yet-upstream patchset (https://github.com/docker/containerd/pull/178):
  + socket-activation-01-vendor.patch
  + socket-activation-02-daemon.patch
  + socket-activation-03-ctr.patch
  * Remove MountFlags=slave since it's not relevant to containerd and might cause
  issues in the future.
  * Update to containerd 0.1.0. This required quite a few fixes.
* Sun Mar 27 2016 asarai@suse.de
  * Add initial packaging of containerd 0.0.5.
  * Add service and sysconfig files.
  * Separately package the client from the server.
  * Install to /usr/sbin.
